/* bloom filter, to record packets received recently
 */

#include <pthread.h>

#include "bloom.h"

typedef unsigned long long int bitset;
#define BLOOM_FILTER_COUNT	16
static bitset old_bloom_filter [BLOOM_FILTER_COUNT]
     = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
static bitset new_bloom_filter [BLOOM_FILTER_COUNT]
     = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
#define MAX_PACKETS_PER_BLOOM_FILTER	32
static char packets_in_new_filter = 0;
static pthread_mutex_t bloom_filter_mutex = PTHREAD_MUTEX_INITIALIZER;

/* returns which bit, in 0..63, to set for this data
 * for bloom hash table number index, in 0..BLOOM_FILTER_COUNT - 1 */
static int bloom_bit (int index, char * data, allnet_data_len data_len)
{
  int bitset_bits = sizeof (bitset) * 8;  /* 64 */
  int bytes_per_index = data_len / BLOOM_FILTER_COUNT + 1;
  int start = index * bytes_per_index;
  if (start >= data_len) start = data_len - 1;
  int after_end = start + bytes_per_index;
  if (after_end > data_len) after_end = data_len;
  int i;
  int hash = 0;
  for (i = start; i < after_end; i++)
    hash += data [i];
  hash = hash % bitset_bits;
  return hash;
}

static int bloom_match_filter (bitset * filter,
                               char * data, allnet_data_len data_len)
{
  int i;
  for (i = 0; i < BLOOM_FILTER_COUNT; i++) {
    int mask = (1 << bloom_bit (i, data, data_len));
    if ((filter [i] & mask) == 0)
      return 0; /* does not match this filter */
  }
  return 1;
}

/* returns 1 if there is a match everywhere in either filter, 0 otherwise */
int bloom_match (char * data, allnet_data_len data_len)
{
  pthread_mutex_lock (&bloom_filter_mutex);
  int result = (bloom_match_filter (new_bloom_filter, data, data_len)) ||
               (bloom_match_filter (old_bloom_filter, data, data_len));
  pthread_mutex_unlock (&bloom_filter_mutex);
  return result;
}

void bloom_add (char * data, allnet_data_len data_len)
{
  pthread_mutex_lock (&bloom_filter_mutex);
  int i;
  for (i = 0; i < BLOOM_FILTER_COUNT; i++)
    new_bloom_filter [i] |= (1 << bloom_bit (i, data, data_len));
  packets_in_new_filter++;
  if (packets_in_new_filter >= MAX_PACKETS_PER_BLOOM_FILTER) {
    for (i = 0; i < BLOOM_FILTER_COUNT; i++) {
      old_bloom_filter [i] = new_bloom_filter [i];
      new_bloom_filter [i] = 0;
    }
    packets_in_new_filter = 0;
  }
  pthread_mutex_unlock (&bloom_filter_mutex);
}
