/* a simple chat app using allnet */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <sys/socket.h>
#include <readline/readline.h>
#include <readline/history.h>

#include "types.h"
#include "messages.h"
#include "api.h"

/* maximum string size for a variety of things */
#define MAGIC_SIZE	141

static char prompt [100];

void receive_fun (void * arg, char * buffer, int bsize,
                 struct allnet_addr * from, struct allnet_route * route,
                 int key_id)
{
  char string [MAGIC_SIZE];
  int size = sizeof (string) - 1;
  if (bsize < size)
    size = bsize;
  memcpy (string, buffer, size);
  string [size] = '\0';

  char name [MAGIC_SIZE] = "unknown";
  if (key_id != 0) 
    name_for_key (key_id, name, sizeof (name));

  printf ("\nfrom %s: %s\n%s", name, string, prompt);
  fflush (stdout);
}

static void my_exit (int port)
{
  allnet_stop (port);
  printf ("goodbye\n");
  exit (0);
}

main (int argc, char ** argv)
{
  /* should be done first, before any use of stdout */
  if (setvbuf (stdout, NULL, _IONBF, 0) != 0) {
    perror ("setvbuf");
    printf ("unable to set stdout to unbuffered mode\n");
  }

  char * my_name = NULL;
  if (argc > 1)
    my_name = argv [1];
  int port = allnet_start (receive_fun, NULL);

  int num_keys = init_key_file ("chat.keys", my_name);
  if (num_keys < 0)
    printf ("unable to open key file, only allowing unencrypted chats\n");

  char * user = NULL;
  int current_key = -1;
  while (1) {
    int request_user = 0;
    if (user != NULL) {
      snprintf (prompt, sizeof (prompt),
                "(0 to quit, 1 to switch user) %s: ", user);
      char * input = readline (prompt);
      if (input == NULL)
        my_exit (port);
      if (*input == '0')
        my_exit (port);
      if (*input == '1')
        request_user = 1;
      else {
        char message [MAGIC_SIZE * 3];
        time_t now_time = time (NULL);
        struct tm * now = localtime (&now_time);
        int send_len = 0;
        if (my_name != NULL)
          send_len = snprintf (message, sizeof (message),
                               "(%s at %02d:%02d:%02d): %s", my_name,
                               now->tm_hour, now->tm_min, now->tm_sec, input);
        else
          send_len = snprintf (message, sizeof (message),
                               "(%02d:%02d:%02d): %s",
                               now->tm_hour, now->tm_min, now->tm_sec, input);
        allnet_send (port, message, send_len, NULL, current_key);
      }
      free (input);
    }
    if ((user == NULL) || (request_user)) {
      strncpy (prompt,
               "enter new name of user or group (empty or 0 to quit): ",
               sizeof (prompt));
      char * new_user = readline (prompt);
      if (new_user == NULL)
        my_exit (port);
      if ((*new_user == '\0') || (*new_user == '0'))
        my_exit (port);
      if (key_for_name (new_user, &current_key, 1) <= 0) {
        printf ("user %s has no key\n", new_user);
        current_key = -1;
      }
      char user_name [MAGIC_SIZE];
      if (name_for_key (current_key, user_name, sizeof (user_name)) > 0)
        printf ("talking to user %s, key ID %d\n", user_name, current_key);
      if (user != NULL) free (user);
      user = new_user;
    }
  }
}

