
#ifndef DEVICE_H
#define DEVICE_H

#include <netpacket/packet.h>   /* defines struct sockaddr_ll */
#include <arpa/inet.h>          /* defines ntohs */

#define ALLNET_WIFI_PROTOCOL (ntohs (0xa119))  /* allN et... */

/* the interface ID is a string meaningful to the system, e.g. "wlan0".
 * if the interface ID is either NULL or the empty string, a random
 * wireless interface is selected.  This works fine with systems that
 * only have one wireless interface.
 *
 * the return value is a positive integer identifying the interface
 * in case of success, or DEVICE_ERROR_INTERFACE_NOT_FOUND otherwise.
 * that integer should be used like a socket in subsequent calls.
 * Each of those calls returns DEVICE_ERROR_BAD_IF_NUMBER if the
 * interface number is not valid, or 0 (DEVICE_CLOSED) if the
 * interface has been turned off.
 */
#define DEVICE_ERROR_INTERFACE_NOT_FOUND	-1
extern int interface_on (char * interface_id);

#define DEVICE_CLOSED				0
#define DEVICE_ERROR_BAD_INTERFACE_NUMBER	-2
extern int interface_off (int if_number);

/* in case of success, the send/receive functions return the number
 * of bytes sent or received.  A return value of 0 means the socket
 * was closed, and a negative value means some other error occurred.
 *
 * if interface_number is 0, sends or receives on any interface that is on.
 * An interface_number of 0 can still give
 * DEVICE_ERROR_BAD_INTERFACE_NUMBER if no interface is turned on.
 */
#define DEVICE_ANY_INTERFACE			0

extern int send_broadcast (int interface_number, char * buffer, int size);

extern int send_unicast (int interface_number, char * buffer, int size,
                        struct sockaddr_ll * addr);

/* waits up to us microseconds, or until a message is received, whichever
 * comes first.  If no message is received, returns DEVICE_RECEIVE_TIMEOUT
 * (not an error).  With 32-bit ints, can wait up to 4000 seconds or so.
 * 
 * addr is filled in with the address of the sender that can be used
 * in a subsequent unicast transmission.
 *
 * if the buffer is not large enough for the incoming message, the
 * message is truncated, and "size" is returned.  In case of truncation,
 * if truncated is not NULL, it is set to the size of the entire packet.
 */

#define DEVICE_RECEIVE_TIMEOUT	-3
#define DEVICE_RECEIVE_US_PER_S	(1000 * 1000)  /* microseconds per second */

extern int receive (int interface_number, char * buffer, int size, int us,
                    struct sockaddr_ll * addr, int * truncated);

#endif /* DEVICE_H */
