/* creates, parses, and prints allnet addresses. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <arpa/inet.h>

#include "types.h"



/* stores an address for sending on the net.
 * if successful, returns the number of bytes used.
 * If bsize is insufficient, returns - the size needed,
 * e.g. -8 for an IPv4 address.
 */
int store_peer (struct allnet_peer * peer, char * buffer, int bsize)
{
  if (bsize < 9)
    return (store_addr (&(peer->peer), buffer, 0) - 8);
  buffer [0] = (peer->provides >> 56) & 0xff;
  buffer [1] = (peer->provides >> 48) & 0xff;
  buffer [2] = (peer->provides >> 40) & 0xff;
  buffer [3] = (peer->provides >> 32) & 0xff;
  buffer [4] = (peer->provides >> 24) & 0xff;
  buffer [5] = (peer->provides >> 16) & 0xff;
  buffer [6] = (peer->provides >>  8) & 0xff;
  buffer [7] = (peer->provides      ) & 0xff;
  int result = store_addr (&(peer->peer), buffer + 8, bsize - 8);
  if (result == 0)
    return 0;
  if (result < 0)
    return result - 8;
  return result + 8;
}

/* returns -1 in case of failure, the number of bytes parsed otherwise.
 * if successful, the peer is stored in *result;
 */
int parse_peer (char * buffer, int bsize, struct allnet_peer * result)
{
  if (bsize < 9)
    return -1;
  result->provides = b8 (buffer);
  int count = parse_addr (buffer + 8, bsize - 8, &(result->peer));
  if (count < 0)
    return count;
  return 8 + count;
}

int peer_to_string (struct allnet_peer * peer, char * string, int ssize)
{
  int result;
  if (peer == NULL) {
    result = snprintf (string, ssize, "(null peer)\n");
    if (result + 1 < ssize) return result + 1; else return - (result + 1);
  }
  result = addr_to_string (&(peer->peer), string, ssize);
  if (result == 0)
    return 0;
  if (result < 0)
    return result - 100; 
  allnet_services s = peer->provides;
  if (s & INTERNET_SERVICE)
    result += snprintf (string + result, ssize - result, " internet,\n");
  if (s & LOCATION_SERVICE)
    result += snprintf (string + result, ssize - result, " location,\n");
  if (s & PHONE_MESSAGE_SERVICE)
    result += snprintf (string + result, ssize - result, " phone message,\n");
  if (result + 1 < ssize) return result + 1; else return - (result + 1);
}

/* prints the address, optionally with a newline at the end (if eof > 0) */
void print_peer (struct allnet_peer * peer, int eol)
{
  int bufsize = 10000;
  char * buffer = malloc (bufsize);
  if (buffer != NULL) {
    int printed = peer_to_string (peer, buffer, bufsize);
    if (printed > 0) {
      if (eol > 0)
        printf ("%s\n", buffer);
      else
        printf ("%s", buffer);
    } else {
      printf ("error printing address\n");
    } 
    free (buffer);
  } else {
    printf ("memory error, unable to print address\n");
  }
}
