
#include <time.h>

#include "types.h"
#include "peer_info.h"

/* if used_peers < SAVED_PEERS, only locations 0..used_peers-1 are in use.
 * otherwise, locations 0..used_peers-1 are in use :)
 */
static struct peer_info peer_infos [SAVED_PEERS];
static int used_peers = 0;
static int last_peer = -1;
static int random_offset = 0;

int stored_peers ()
{
  return used_peers;
}

/* if the peer is already saved, does nothing */ 
void add_peer (struct allnet_peer * peer)
{
  int i;
  for (i = 0 ; i < used_peers; i++) {
    if (same_addr (&(peer->peer), &(peer_infos [i].peer.peer)))
      return;
  }
  struct peer_info * new_peer;
  if (used_peers < SAVED_PEERS) {
    new_peer = peer_infos + used_peers;
    last_peer = used_peers;
    used_peers++;
  } else {
    last_peer = (last_peer + 1) % SAVED_PEERS;
    /* overwrite the oldest peer in the list, which is just after last_peer */
    new_peer = peer_infos + last_peer;
  }
  new_peer->peer = *peer;
  time_t now = time (NULL);
  new_peer->last_sent = 0;
  new_peer->last_received = 0;
}

/* record that a hello was just received from this peer */
void received_now (struct allnet_peer * peer)
{
  int i;
  for (i = 0 ; i < used_peers; i++) {
    if (same_addr (&(peer->peer), &(peer_infos [i].peer.peer)))
      time (&(peer_infos [i].last_received));
  }
}

static int satisfies_conditions (int index, int must_be_ip,
                                 int seconds_since_last_sent)
{
  struct peer_info * p = peer_infos + index;
  time_t now;
  time (&now);
  if ((must_be_ip) &&
      ((p->peer.peer.discrim != ALLNET_IP4_ADDR_ID) &&
       (p->peer.peer.discrim != ALLNET_IP4_ADDR_ID)))
    /* not the right kind of address */
    return 0;
  if ((seconds_since_last_sent > 0) &&
      (p->last_sent != 0) &&
      ((now - p->last_sent) < seconds_since_last_sent))
    /* not enough time since last sent */
    return 0;
  /* update the "last_sent" time, assuming we will send now */
  if (seconds_since_last_sent > 0)
    p->last_sent = now;
  return 1;
}

/* returns the number of peers placed into *peer, which will not
 * in any case exceed max_peers, but may be fewer
 *
 * if must_be_ip, only peers with IP addresses (v4 or v6) are returned.
 * if seconds_since_last_sent > 0, only peers with last_sent > the parameter
 * are returned, and their send time is updated to now
 */
extern int random_peers (struct peer_info * peer, int max_peers,
                         int must_be_ip, int seconds_since_last_sent)
{
  int i = 0;
  int limit = max_peers;
  if (max_peers >= used_peers)  /* return all peers */
    limit = used_peers;
  while (i < limit) {
    int read_index = (i + random_offset) % used_peers;
    if (satisfies_conditions (i, must_be_ip, seconds_since_last_sent)) {
      peer [i] = peer_infos [read_index];
      i++;
    } else {
      limit--;
      random_offset++;  /* so we look at a new peer without incrementing i */
    }
  }
  random_offset++;  /* return a slightly different set next time */
  return limit;
}

