/* creates, parses, and prints hello messages. */

/* allnet daemon, runs independently and forwards messages based on priority.
 *
 * this daemon occasionally listens and sends on wireless (protocol
 * number 0xA119) and constantly listens and occasionally sends allnet
 * traffic on UDP port 0xA119.
 *
 * this daemon also communicates with any local allnet clients on TCP
 * port 0xA11E.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "types.h"
#include "messages.h"

int store_route (struct allnet_route * route, char * buffer, int bsize)
{
  if (bsize > 0)
    buffer [0] = b1 (route->num_addrs);
  if (b1 (route->num_addrs) == 0)
    return ((bsize > 0) ? 1 : -1);
  if (bsize > 1)
    buffer [1] = b1 (route->used);

  int needed = 2;
  int i;
  int num_addrs = b1 (route->num_addrs);

  for (i = 0; i < num_addrs; i++) {
    int result;
    if (bsize > needed)  /* go ahead and store the address */
      result = store_addr (route->addrs + i, buffer + needed, bsize - needed);
    else  /* just compute the size */
      result = store_addr (route->addrs + i, buffer, 0);
    if (result == 0) {
      printf ("unable to store route, failed at address %d\n", i);
      return 0;
    }
    if (result > 0)
      needed += result;
    else
      needed -= result;
  }
  if (bsize >= needed)
    return needed;
  else
    return -needed;
}

int parse_route (char * buffer, int bsize, struct allnet_route * result)
{
  result->num_addrs [0] = 0;
  result->used [0] = 0;
  result->addrs = NULL;
  if (bsize < 1)
    return -1;

  int parsed = 1;
  int num_addrs = b1 (buffer);
  result->num_addrs [0] = num_addrs;
  if (bsize > 1) {
    result->used [0] = b1 (buffer + 1);
    parsed = 2;
  }
  if ((bsize > 2) && (result->num_addrs > 0)) {
    result->addrs = malloc (sizeof (struct allnet_addr) * num_addrs);
    if (result->addrs == NULL) { perror ("malloc"); return -1; }
    buffer += parsed;
    bsize -= parsed;
    int i = 0;
    while ((i < num_addrs) && (bsize > 0)) {
      int p = parse_addr (buffer, bsize, result->addrs + i);
      if (p <= 0) {
        free (result->addrs);
        result->addrs = NULL;
        return -1;
      }
      parsed += p;
      buffer += p;
      bsize -= p;
      i++;
    }
  }
  return parsed;
}

int route_to_string (struct allnet_route * route, char * string, int ssize)
{
  int num_addrs = b1 (route->num_addrs);
  int used = b1 (route->used);
  int printed = snprintf (string, ssize, "route with %d addresses", num_addrs);
  if (num_addrs == 0) {
    if (printed + 1 >= ssize)
      return printed + 1;
    else
      return - (printed + 1);
  }

  int r;
  if (printed + 1 >= ssize)
    r = snprintf (string + printed, ssize - printed, ", %d used", used);
  else
    r = snprintf (string, ssize, ", %d used", used);
  if (r < 0) { printf ("error in route_to_string\n"); return 0; }
  printed += r;

  int i;
  for (i = 0; i < num_addrs; i++) {
    if (printed + 1 >= ssize)
      r = addr_to_string (route->addrs + i, string + printed, ssize - printed);
    else
      r = addr_to_string (route->addrs + i, string, ssize);
    if (r < 0) { printf ("error (addr) in route_to_string\n"); return 0; }
    printed += r;
  }
  if (printed + 1 >= ssize)
    return printed + 1;
  else
    return - (printed + 1);
}

void print_route (struct allnet_route * route, int eol)
{
  if (route != NULL) {
    printf ("route with %d addresses, of which %d used",
            b1 (route->num_addrs), b1 (route->used));
    if (eol > 0)
      printf ("\n");
    else
      printf (": ");
    int i;
    for (i = 0; i < b1 (route->num_addrs); i++) {
      printf ("%d: ", i + 1);
      print_addr (route->addrs + i, eol);
    }
  } else {
    printf ("(null route)");
    if (eol > 0)
      printf ("\n");
  }
}

