
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define max(a, b) (((a) > (b)) ? (a) : (b))

/* failed -- either crash, or set *fail */
static void fail_or_set (int new_size, char * desc, int * fail)
{
  /* failed -- either crash, or set *fail */
  printf ("%s: unable to allocate %d bytes for %s\n",
          ((*fail) ? "error" : "warning"), new_size, desc);
  if ((fail == NULL) || (*fail)) {
    /* exit (1);  do null pointer dereferencing instead, so gdb can catch it */
    fail = NULL;  /* null pointer dereferencing at the next step */
  }
  *fail = 1;
}

/* ensure_size ensures that the buffer given in old_value has the size
 * requested by *buffer_size.  old_value may be NULL for a new allocation.
 *
 * if successful, returns the new memory which has the desired size,
 * setting *buffer_size to the size of the returned buffer, which may
 * be more than new_size.
 *
 * a typical calling sequence would be:
 *   static char * buffer = NULL;
 *   static int buffer_size = 0;
 *   int wanted = ...
 *   int fail = 0;
 *   buffer = ensure_size (&buffer_size, wanted, buffer, "foo", &fail);
 *   if (fail) ...
 *
 * if the call fails, prints an error message and, if fail is not
 * NULL and *fail is 0, sets *fail to 1 and does not change *buffer_size.
 * if fail is NULL or if *fail != 0, crashes after printing.
 */
void * ensure_size (int * buffer_size, int new_size,
                    char * old_value, char * desc, int * fail)
{
  int old_size = *buffer_size;
  int zero_start = (NULL == old_value) || (*buffer_size == 0LL);
  if ((! zero_start) && (new_size <= *buffer_size))
    return old_value;         /* this should be the common case */
  /* old value is null or new_size > old_size */
  if ((NULL == old_value) && (0LL != *buffer_size))
    printf ("warning: %s buffer size is %d, but buffer is NULL\n",
            desc, *buffer_size);
  /* increment by at least 50% */
  new_size = max (new_size, old_size + old_size / 2 + 1);
  /* make new size the next larger multiple of 8 */
  new_size = (new_size + 7LL) & (~ (0x7LL));
  if (new_size > 10000000LL)
    printf ("%salloc (%d)\n", ((zero_start) ? "m" : "re"), new_size);
  char * result;
  if (zero_start)
    result = malloc (new_size);
  else
    result = realloc (old_value, new_size);
  if (NULL != result) {       /* re/allocation was OK, life is good */
    *buffer_size = new_size;
    return result;
  }
  /* failed re/allocation */
  fail_or_set (new_size, desc, fail);
  /* return the old memory, which may still be useful */
  return old_value;
}

/* allocates the desired memory.  Failure is as in the previous case.
   If the call fails but returns, it returns null.  */
void * new_buf (int new_size, char * desc, int * fail)
{
  int allocated = 0;
  return ensure_size (&allocated, new_size, NULL, desc, fail);
}

