/* ensure_size ensures that the buffer given in old_value has the size
 * requested by *buffer_size.  old_value may be NULL for a new allocation.
 *
 * if successful, returns the new memory which has the desired size,
 * setting *buffer_size to the size of the returned buffer, which may
 * be more than new_size.
 *
 * a typical calling sequence would be:
 *   static char * buffer = NULL;
 *   static int buffer_size = 0;
 *   int wanted = ...
 *   int fail = 0;
 *   buffer = ensure_size (&buffer_size, wanted, buffer, "foo", &fail);
 *   if (fail) ...
 *
 * if the call fails, prints an error message and, if fail is not
 * NULL and *fail is 0, sets *fail to 1 and does not change *buffer_size.
 * if fail is NULL or if *fail != 0, crashes after printing.
 */
extern void * ensure_size (int * buffer_size, int new_size,
                           char * old_value, char * desc, int * fail);

/* allocates the desired memory.  Failure is as in the previous case.
   If the call fails but returns, it returns null.  */
extern void * new_buf (int new_size, char * desc, int * fail);


